<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Author: Stefan Neufeind <pear.neufeind@speedpartner.de>              |
// +----------------------------------------------------------------------+
//
// $Id$

/**
* Testcases for Punycode-implementation
*
* @author   Stefan Neufeind <pear.neufeind@speedpartner.de>
* @package  I18N_Punycode
* @category i18n
* @license  The PHP License, version 2.02
*/

require_once 'PHPUnit.php'; 
require_once 'I18N/Punycode.php';

/**
* Testcases-class
*
* @author   Stefan Neufeind <pear.neufeind@speedpartner.de>
* @package  I18N_Punycode
* @access   public
*/
class I18N_Punycode_Test extends PHPUnit_TestCase{
    function I18N_Punycode_Test($name)
    {
        $this->PHPUnit_TestCase($name);
    }

    function setUp(){}
    function tearDown(){}

    /**
    * Perform encoding-tests using I18N_Punycode_ASCII
    *
    * !! Note that the ASCII-handling only supports a limited set of the
    * Punycode-implementation. If possible it's generally better to use Unicode.
    *
    * @access public
    */
    function testAsciiEncode()
    {
        $obj = & I18N_Punycode_ASCII::factory();
        $this->assertEquals("xn--grtnerei-mller-5hb70b.de", $obj->encode("g\xe4rtnerei-m\xfcller.de", "ISO-8859-1"));
        $this->assertEquals("xn--grtnerei-mller-5hb70b.de", $obj->encode("g\xc4rtnereI-m\xdcller.de", "ISO-8859-1")); // because of nameprep
        $this->assertEquals("xn--schne-grsse-tfb7f.de",     $obj->encode("sch\xf6ne-gr\xfce.de", "ISO-8859-1"));
        $this->assertEquals("xn--4ca.de",                   $obj->encode("\xe4.de", "ISO-8859-1"));
        $this->assertEquals("ss.de",                        $obj->encode("\xdf.de", "ISO-8859-1")); // because of nameprep
    }

    /**
    * Perform encoding-tests using I18N_Punycode_Unicode
    *
    * Here we use strings in ISO-8859-1 format. But internally they are converted and
    * processed as Unicode.
    *
    * @access public
    */
    function testUnicodeEncode_ISO_8859_1()
    {
        $obj = & I18N_Punycode_Unicode::factory('mbstring');
        $this->assertEquals("xn--grtnerei-mller-5hb70b.de", $obj->encode("g\xe4rtnerei-m\xfcller.de", "ISO-8859-1"));
        $this->assertEquals("xn--grtnerei-mller-5hb70b.de", $obj->encode("g\xc4rtnereI-m\xdcller.de", "ISO-8859-1")); // because of nameprep
        $this->assertEquals("xn--schne-grsse-tfb7f.de",     $obj->encode("sch\xf6ne-gr\xfce.de", "ISO-8859-1"));
        $this->assertEquals("xn--4ca.de",                   $obj->encode("\xe4.de", "ISO-8859-1"));
        $this->assertEquals("ss.de",                        $obj->encode("\xdf.de", "ISO-8859-1")); // because of nameprep
    }

    /**
    * Perform decoding-tests using I18N_Punycode_ASCII
    *
    * !! Note that the ASCII-handling only supports a limited set of the
    * Punycode-implementation. If possible it's generally better to use Unicode.
    *
    * @access public
    */
    function testAsciiDecode()
    {
        $obj = & I18N_Punycode_ASCII::factory();
        $this->assertEquals("g\xe4rtnerei-m\xfcller.de",    $obj->decode("xn--grtnerei-mller-5hb70b.de", "ISO-8859-1"));
        $this->assertEquals("sch\xf6ne-gr\xfcsse.de",       $obj->decode("xn--schne-grsse-tfb7f.de", "ISO-8859-1")); // because of nameprep
        $this->assertEquals("\xe4.de",                      $obj->decode("xn--4ca.de", "ISO-8859-1"));
        $this->assertEquals("ss.de",                        $obj->decode("ss.de", "ISO-8859-1")); // because of nameprep
    }

    /**
    * Perform decoding-tests using I18N_Punycode_Unicode
    *
    * Here we use strings in ISO-8859-1 format. But internally they are converted and
    * processed as Unicode.
    *
    * @access public
    */
    function testUnicodeDecode_ISO_8859_1()
    {
        $obj = & I18N_Punycode_Unicode::factory('mbstring');
        $this->assertEquals("g\xe4rtnerei-m\xfcller.de",    $obj->decode("xn--grtnerei-mller-5hb70b.de", "ISO-8859-1"));
        $this->assertEquals("sch\xf6ne-gr\xfcsse.de",       $obj->decode("xn--schne-grsse-tfb7f.de", "ISO-8859-1")); // because of nameprep
        $this->assertEquals("\xe4.de",                      $obj->decode("xn--4ca.de", "ISO-8859-1"));
        $this->assertEquals("ss.de",                        $obj->decode("ss.de", "ISO-8859-1")); // because of nameprep
    }
    
    /**
    * Expand an ASCII-string into UCS-4 format.
    *
    * Expanding ASCII to UCS-4 is simple. Just add three \x00 before each char/byte.
    *
    * @access private
    */
    function _expandASCIIToUCS4($ascii)
    {
        $output = "";
        for($counter=0; $counter<strlen($ascii); $counter++) {
            $output .= "\x00\x00\x00".$ascii[$counter];
        }
        return ($output);
    }

    /**
    * Perform encoding-tests using I18N_Punycode_Unicode; tests taken from RFC3492
    *
    * The test-strings are in various languages, including e.g. Chinese or Cyrillic.
    * They function as tests for "exotic" input-strings, and seem to be a very wide
    * range of tests.
    *
    * @access public
    */
    function testUnicodeEncode_RFC3492_UCS4()
    {
        $obj = & I18N_Punycode_Unicode::factory('mbstring');

        // (A) Arabic (Egyptian)
        $input           = "\x00\x00\x06\x44\x00\x00\x06\x4A\x00\x00\x06\x47\x00\x00\x06\x45\x00\x00\x06\x27\x00\x00\x06\x28\x00\x00\x06\x2A\x00\x00\x06\x43\x00\x00\x06\x44".
                           "\x00\x00\x06\x45\x00\x00\x06\x48\x00\x00\x06\x34\x00\x00\x06\x39\x00\x00\x06\x31\x00\x00\x06\x28\x00\x00\x06\x4A\x00\x00\x06\x1F";
        $expected_output = "xn--egbpdaj6bu4bxfgehfvwxn";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (B) Chinese (simplified)
        $input           = "\x00\x00\x4E\xD6\x00\x00\x4E\xEC\x00\x00\x4E\x3A\x00\x00\x4E\xC0\x00\x00\x4E\x48\x00\x00\x4E\x0D\x00\x00\x8B\xF4\x00\x00\x4E\x2D\x00\x00\x65\x87";
        $expected_output = "xn--ihqwcrb4cv8a8dqg056pqjye";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (C) Chinese (traditional)
        $input           = "\x00\x00\x4E\xD6\x00\x00\x50\x11\x00\x00\x72\x32\x00\x00\x4E\xC0\x00\x00\x9E\xBD\x00\x00\x4E\x0D\x00\x00\x8A\xAA\x00\x00\x4E\x2D\x00\x00\x65\x87";
        $expected_output = "xn--ihqwctvzc91f659drss3x8bo0yb";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));


        // (D) Czech: Pro<ccaron>prost<ecaron>nemluv<iacute><ccaron>esky
        $input           = "\x00\x00\x00\x50\x00\x00\x00\x72\x00\x00\x00\x6F\x00\x00\x01\x0D\x00\x00\x00\x70\x00\x00\x00\x72\x00\x00\x00\x6F\x00\x00\x00\x73\x00\x00\x00\x74".
                           "\x00\x00\x01\x1B\x00\x00\x00\x6E\x00\x00\x00\x65\x00\x00\x00\x6D\x00\x00\x00\x6C\x00\x00\x00\x75\x00\x00\x00\x76\x00\x00\x00\xED\x00\x00\x01\x0D".
                           "\x00\x00\x00\x65\x00\x00\x00\x73\x00\x00\x00\x6B\x00\x00\x00\x79";
        $expected_output = "xn--proprostnemluvesky-uyb24dma41a";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (E) Hebrew
        $input           = "\x00\x00\x05\xDC\x00\x00\x05\xDE\x00\x00\x05\xD4\x00\x00\x05\xD4\x00\x00\x05\xDD\x00\x00\x05\xE4\x00\x00\x05\xE9\x00\x00\x05\xD5\x00\x00\x05\xD8".
                           "\x00\x00\x05\xDC\x00\x00\x05\xD0\x00\x00\x05\xDE\x00\x00\x05\xD3\x00\x00\x05\xD1\x00\x00\x05\xE8\x00\x00\x05\xD9\x00\x00\x05\xDD\x00\x00\x05\xE2".
                           "\x00\x00\x05\xD1\x00\x00\x05\xE8\x00\x00\x05\xD9\x00\x00\x05\xEA";
        $expected_output = "xn--4dbcagdahymbxekheh6e0a7fei0b";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (F) Hindi (Devanagari)
        $input           = "\x00\x00\x09\x2F\x00\x00\x09\x39\x00\x00\x09\x32\x00\x00\x09\x4B\x00\x00\x09\x17\x00\x00\x09\x39\x00\x00\x09\x3F\x00\x00\x09\x28\x00\x00\x09\x4D".
                           "\x00\x00\x09\x26\x00\x00\x09\x40\x00\x00\x09\x15\x00\x00\x09\x4D\x00\x00\x09\x2F\x00\x00\x09\x4B\x00\x00\x09\x02\x00\x00\x09\x28\x00\x00\x09\x39".
                           "\x00\x00\x09\x40\x00\x00\x09\x02\x00\x00\x09\x2C\x00\x00\x09\x4B\x00\x00\x09\x32\x00\x00\x09\x38\x00\x00\x09\x15\x00\x00\x09\x24\x00\x00\x09\x47".
                           "\x00\x00\x09\x39\x00\x00\x09\x48\x00\x00\x09\x02";
        $expected_output = "xn--i1baa7eci9glrd9b2ae1bj0hfcgg6iyaf8o0a1dig0cd";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (G) Japanese (kanji and hiragana)
        $input           = "\x00\x00\x30\x6A\x00\x00\x30\x5C\x00\x00\x30\x7F\x00\x00\x30\x93\x00\x00\x30\x6A\x00\x00\x65\xE5\x00\x00\x67\x2C\x00\x00\x8A\x9E\x00\x00\x30\x92".
                           "\x00\x00\x8A\x71\x00\x00\x30\x57\x00\x00\x30\x66\x00\x00\x30\x4F\x00\x00\x30\x8C\x00\x00\x30\x6A\x00\x00\x30\x44\x00\x00\x30\x6E\x00\x00\x30\x4B";
        $expected_output = "xn--n8jok5ay5dzabd5bym9f0cm5685rrjetr6pdxa";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (H) Korean (Hangul syllables)
        $input           = "\x00\x00\xC1\x38\x00\x00\xAC\xC4\x00\x00\xC7\x58\x00\x00\xBA\xA8\x00\x00\xB4\xE0\x00\x00\xC0\xAC\x00\x00\xB7\x8C\x00\x00\xB4\xE4\x00\x00\xC7\x74".
                           "\x00\x00\xD5\x5C\x00\x00\xAD\x6D\x00\x00\xC5\xB4\x00\x00\xB9\x7C\x00\x00\xC7\x74\x00\x00\xD5\x74\x00\x00\xD5\x5C\x00\x00\xB2\xE4\x00\x00\xBA\x74".
                           "\x00\x00\xC5\xBC\x00\x00\xB9\xC8\x00\x00\xB0\x98\x00\x00\xC8\x8B\x00\x00\xC7\x44\x00\x00\xAE\x4C";
        $expected_output = "xn--989aomsvi5e83db1d2a355cv1e0vak1dwrv93d5xbh15a0dt30a5jpsd879ccm6fea98c";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (I) Russian (Cyrillic)
        $input           = "\x00\x00\x04\x3F\x00\x00\x04\x3E\x00\x00\x04\x47\x00\x00\x04\x35\x00\x00\x04\x3C\x00\x00\x04\x43\x00\x00\x04\x36\x00\x00\x04\x35\x00\x00\x04\x3E".
                           "\x00\x00\x04\x3D\x00\x00\x04\x38\x00\x00\x04\x3D\x00\x00\x04\x35\x00\x00\x04\x33\x00\x00\x04\x3E\x00\x00\x04\x32\x00\x00\x04\x3E\x00\x00\x04\x40".
                           "\x00\x00\x04\x4F\x00\x00\x04\x42\x00\x00\x04\x3F\x00\x00\x04\x3E\x00\x00\x04\x40\x00\x00\x04\x43\x00\x00\x04\x41\x00\x00\x04\x41\x00\x00\x04\x3A".
                           "\x00\x00\x04\x38";
        $expected_output = "xn--b1abfaaepdrnnbgefbadotcwatmq2g4l";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (J) Spanish: Porqu<eacute>nopuedensimplementehablarenEspa<ntilde>ol 
        $input           = "\x00\x00\x00\x50\x00\x00\x00\x6F\x00\x00\x00\x72\x00\x00\x00\x71\x00\x00\x00\x75\x00\x00\x00\xE9\x00\x00\x00\x6E\x00\x00\x00\x6F\x00\x00\x00\x70".
                           "\x00\x00\x00\x75\x00\x00\x00\x65\x00\x00\x00\x64\x00\x00\x00\x65\x00\x00\x00\x6E\x00\x00\x00\x73\x00\x00\x00\x69\x00\x00\x00\x6D\x00\x00\x00\x70".
                           "\x00\x00\x00\x6C\x00\x00\x00\x65\x00\x00\x00\x6D\x00\x00\x00\x65\x00\x00\x00\x6E\x00\x00\x00\x74\x00\x00\x00\x65\x00\x00\x00\x68\x00\x00\x00\x61".
                           "\x00\x00\x00\x62\x00\x00\x00\x6C\x00\x00\x00\x61\x00\x00\x00\x72\x00\x00\x00\x65\x00\x00\x00\x6E\x00\x00\x00\x45\x00\x00\x00\x73\x00\x00\x00\x70".
                           "\x00\x00\x00\x61\x00\x00\x00\xF1\x00\x00\x00\x6F\x00\x00\x00\x6C";
        $expected_output = "xn--porqunopuedensimplementehablarenespaol-fmd56a";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (K) Vietnamese: T<adotbelow>isaoh<odotbelow>kh<ocirc>ngth<ecirchookabove>ch<ihookabove>n<oacute>iti<ecircacute>ngVi<ecircdotbelow>t
        $input           = "\x00\x00\x00\x54\x00\x00\x1E\xA1\x00\x00\x00\x69\x00\x00\x00\x73\x00\x00\x00\x61\x00\x00\x00\x6F\x00\x00\x00\x68\x00\x00\x1E\xCD\x00\x00\x00\x6B".
                           "\x00\x00\x00\x68\x00\x00\x00\xF4\x00\x00\x00\x6E\x00\x00\x00\x67\x00\x00\x00\x74\x00\x00\x00\x68\x00\x00\x1E\xC3\x00\x00\x00\x63\x00\x00\x00\x68".
                           "\x00\x00\x1E\xC9\x00\x00\x00\x6E\x00\x00\x00\xF3\x00\x00\x00\x69\x00\x00\x00\x74\x00\x00\x00\x69\x00\x00\x1E\xBF\x00\x00\x00\x6E\x00\x00\x00\x67".
                           "\x00\x00\x00\x56\x00\x00\x00\x69\x00\x00\x1E\xC7\x00\x00\x00\x74";
        $expected_output = "xn--tisaohkhngthchnitingvit-kjcr8268qyxafd2f1b9g";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // from here on we use names of Japanese music artists, song titles, and TV programs

        // (L) 3<nen>B<gumi><kinpachi><sensei> 
        $input           = "\x00\x00\x00\x33\x00\x00\x5E\x74\x00\x00\x00\x42\x00\x00\x7D\x44\x00\x00\x91\xD1\x00\x00\x51\x6B\x00\x00\x51\x48\x00\x00\x75\x1F";
        $expected_output = "xn--3b-ww4c5e180e575a65lsy2b";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (M) <amuro><namie>-with-SUPER-MONKEYS
        $input           = "\x00\x00\x5B\x89\x00\x00\x5B\xA4\x00\x00\x59\x48\x00\x00\x7F\x8E\x00\x00\x60\x75\x00\x00\x00\x2D\x00\x00\x00\x77\x00\x00\x00\x69\x00\x00\x00\x74".
                           "\x00\x00\x00\x68\x00\x00\x00\x2D\x00\x00\x00\x53\x00\x00\x00\x55\x00\x00\x00\x50\x00\x00\x00\x45\x00\x00\x00\x52\x00\x00\x00\x2D\x00\x00\x00\x4D".
                           "\x00\x00\x00\x4F\x00\x00\x00\x4E\x00\x00\x00\x4B\x00\x00\x00\x45\x00\x00\x00\x59\x00\x00\x00\x53";
        $expected_output = "xn---with-super-monkeys-pc58ag80a8qai00g7n9n";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (N) Hello-Another-Way-<sorezore><no><basho> 
        $input           = "\x00\x00\x00\x48\x00\x00\x00\x65\x00\x00\x00\x6C\x00\x00\x00\x6C\x00\x00\x00\x6F\x00\x00\x00\x2D\x00\x00\x00\x41\x00\x00\x00\x6E\x00\x00\x00\x6F".
                           "\x00\x00\x00\x74\x00\x00\x00\x68\x00\x00\x00\x65\x00\x00\x00\x72\x00\x00\x00\x2D\x00\x00\x00\x57\x00\x00\x00\x61\x00\x00\x00\x79\x00\x00\x00\x2D".
                           "\x00\x00\x30\x5D\x00\x00\x30\x8C\x00\x00\x30\x5E\x00\x00\x30\x8C\x00\x00\x30\x6E\x00\x00\x58\x34\x00\x00\x62\x40";
        $expected_output = "xn--hello-another-way--fc4qua05auwb3674vfr0b";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (O) <hitotsu><yane><no><shita>2 
        $input           = "\x00\x00\x30\x72\x00\x00\x30\x68\x00\x00\x30\x64\x00\x00\x5C\x4B\x00\x00\x68\x39\x00\x00\x30\x6E\x00\x00\x4E\x0B\x00\x00\x00\x32";
        $expected_output = "xn--2-u9tlzr9756bt3uc0v";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (P) Maji<de>Koi<suru>5<byou><mae> 
        $input           = "\x00\x00\x00\x4D\x00\x00\x00\x61\x00\x00\x00\x6A\x00\x00\x00\x69\x00\x00\x30\x67\x00\x00\x00\x4B\x00\x00\x00\x6F\x00\x00\x00\x69\x00\x00\x30\x59".
                           "\x00\x00\x30\x8B\x00\x00\x00\x35\x00\x00\x79\xD2\x00\x00\x52\x4D";
        $expected_output = "xn--majikoi5-783gue6qz075azm5e";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (Q) <pafii>de<runba> 
        $input           = "\x00\x00\x30\xD1\x00\x00\x30\xD5\x00\x00\x30\xA3\x00\x00\x30\xFC\x00\x00\x00\x64\x00\x00\x00\x65\x00\x00\x30\xEB\x00\x00\x30\xF3\x00\x00\x30\xD0";
        $expected_output = "xn--de-jg4avhby1noc0d";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));

        // (R) <sono><supiido><de> 
        $input           = "\x00\x00\x30\x5D\x00\x00\x30\x6E\x00\x00\x30\xB9\x00\x00\x30\xD4\x00\x00\x30\xFC\x00\x00\x30\xC9\x00\x00\x30\x67";
        $expected_output = "xn--d9juau41awczczp";
        $this->assertEquals($this->_expandASCIIToUCS4($expected_output), $obj->encode($input, "UCS-4"));
    }

}

$s = &new PHPUnit_TestSuite("I18N_Punycode_Test");
$r = PHPUnit::run($s);
echo $r->toString(); 

?>